﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Paying.Payment.Selector', {
    extend: 'System.Control.EntitySelector',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.Control.Kivii.Finances.Paying.Payment.Selector',
    dataSource: '/Restful/Kivii.Finances.Entities.Payment/PayQuery.json',
    filter: {
    	OwnerKvid:KiviiContext.CurrentMember.DepartmentKvid
    },
    selModel: false,
    store: {
        autoLoad: true,
        pageSize: 50,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    tbar: [{
        xtype: 'searchField',
        reference: 'searchfield',
        flex: 1,
        labelWidth: 60,
        queryField: 'PayerName',
        emptyText: '模糊搜索付款单位'
    }],
    columnLines: true,
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            width: 210
        }, {
            text: '发生金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 110
        }, {
            text: '可用金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: '_renderAmount',
            width: 110
        },  {
            text: '开票金额',
            align: 'right',
            dataIndex: 'AmountInvoice',
            renderer: '_renderAmountInvoice',
            width: 100
        },{
            xtype: 'datecolumn',
            text: '发生日期',
            format: 'Y-m-d',
            dataIndex: 'OperateTime',
            width: 100
        }, {
            text: '付款类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 100
        },{
            text: '付款银行',
            dataIndex: 'PayerAccountName',
            width: 280
        }, {
            text: '付款账号',
            dataIndex: 'PayerAccountSerialNumber',
            width: 180

        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 150
        }, {
            text: '备注',
            dataIndex: 'Remark',
            flex: 1
        }]
    },
    setOwner: function(_entity) {
        let _view = this;
        let _refs = _view.getReferences();
        let _search = _refs.searchfield;
        _search.setValue(_entity.PayerName);
        let _searchFilter = {};
        _searchFilter.QueryKeys = _search.queryField;
        _searchFilter.QueryValues = _search.getRawValue();
        let _fields = _view.query('field[queryField]');
        for (let _i in _fields) {
            _field = _fields[_i];
            if (_field === _view) continue;
            if (!_field.queryField) continue;
            _value = _field.getRawValue();
            _searchFilter[_field.queryField] = _value;
        }
        _view.setSearchFilter(_searchFilter);
        // _view.setFilter({
        // 	OwnerKvid:KiviiContext.CurrentMember.DepartmentKvid
        // });
        _view.reload && _view.reload();
    },
    controller: {
        _renderType: function(_value, _meta, _record, _rowIndex) {
        	_meta.style = 'background-color:#B4EEB4';
            if (_value === 'UnBiz') {
            	_meta.style = 'background-color:#F0E68C';
                return '非业务';
            }
            if (_value === 'Refund') {
            	_meta.style = 'background-color:#FFC0CB';
                return '退款';
            }
            if (_value === 'Split') {
                return '业务认领';
            }
            if (_value === 'Pos') {
                return '刷卡';
            }
            if (_value === 'Cash') {
                return '现金';
            }
            if (_value === 'AliPay') {
                return '支付宝';
            }
            if (_value === 'WeChat') {
                return '微信支付';
            }
            return _value;
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            let _canBeUsedAmount = _value - _record.data.AmountUsed;
            _meta.style = 'background-color:#B4EEB4';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_canBeUsedAmount);
        },
        _renderAmountInvoice: function(_value, _meta, _record, _rowIndex) {
            if (_value !== _record.data.Amount) {
                _meta.style = 'background-color:#F0E68C';
            }
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
    }
});