﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{

    [Api(Description = "发票作废")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceOffset : RestfulExecution<Invoice>
    {
        public Guid Kvid { get; set; }
        public List<Guid> Kvids { get; set; }

        public string Remark { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()) throw new Exception("请传入要作废的发票Kvid!");
            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);
            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var invoices = conn.SelectByIds<Invoice>(Kvids);
            if (invoices.IsNullOrEmpty()) throw new Exception("未找到所选发票信息!");
            if (invoices.Exists(o => o.BizKvid != Guid.Empty || o.OffsetKvid != Guid.Empty || o.RootKvid != o.Kvid || o.Amount <= 0)) throw new Exception("存在不正确的发票类型");
            if (invoices.Count != Kvids.Count) throw new Exception("所选发票信息与查询后结果不一致!");
            var relationInvoices = conn.Select<Invoice>(o => Sql.In(o.ParentKvid, Kvids) && o.OffsetKvid == Guid.Empty && o.Type == "Relation");
            var existApplies = conn.Select<InvoiceApply>(o => o.OperateType == InvoiceApplyType.Related && Sql.In(o.RootKvid, invoices.ConvertAll(p => p.ApplyKvid)));
            List<Payment> existPayments = null;
            List<Payment> splitPayments = null;
            List<Invoice> offsets = new List<Invoice>();
            if (!relationInvoices.IsNullOrEmpty())
            {
                existPayments = conn.Select<Payment>(o => Sql.In(o.Kvid, relationInvoices.ConvertAll(p => p.BizKvid)));//Type:Bank,Pos,Cash,WeChat...
                if (!existPayments.IsNullOrEmpty())
                {
                    splitPayments = conn.Select<Payment>(o => o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Split && Sql.In(o.ParentKvid, existPayments.ConvertAll(p => p.Kvid)));
                    foreach (var item in existPayments)
                    {
                        var relations = relationInvoices.Where(o => o.BizKvid == item.Kvid).ToList();
                        if (!relations.IsNullOrEmpty())
                        {
                            item.AmountInvoice -= relations.Sum(o => o.Amount);
                            if (item.AmountInvoice < 0) item.AmountInvoice = 0;
                            item.AddOnlyProperties(o => o.AmountInvoice);
                        }
                        if (!splitPayments.IsNullOrEmpty())//要是所选Payment存在拆分项,满足条件就要更新AmountInvoice
                        {
                            decimal amountInvoice = 0;
                            amountInvoice.PopulateWith(item.AmountInvoice);// = payment.AmountInvoice;
                            foreach (var split in splitPayments)
                            {
                                if (amountInvoice <= 0) split.AmountInvoice = 0;
                                else split.AmountInvoice = split.Amount >= amountInvoice ? amountInvoice : split.Amount;
                                split.AddOnlyProperties(o => o.AmountInvoice);
                                amountInvoice -= split.AmountInvoice;
                            }
                        }
                    }
                }
                foreach (var item in relationInvoices)
                {
                    var offset = item.Offset($"发票作废:{Remark}");
                    offsets.Add(offset);
                }
            }
            foreach (var item in invoices)
            {
                var offset = item.Offset($"发票作废:{Remark}");
                offsets.Add(offset);
            }
            var rtns = new RestfulUpdateResponse<Invoice>();
            rtns.Results = new List<Invoice>();

            var trans = conn.OpenTransaction();
            try
            {
                foreach (var item in invoices)
                {
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);
                }
                //删除关联Payment的发票数据
                if (!relationInvoices.IsNullOrEmpty())
                {
                    foreach (var item in relationInvoices)
                    {
                        conn.UpdateOnly(item);
                    }
                }
                foreach (var item in offsets)
                {
                    conn.Insert(item);
                }
                //更新已关联的payment的AmountInvoice
                if (!existPayments.IsNullOrEmpty())
                {
                    foreach (var item in existPayments)
                    {
                        if (item.OnlyProperties.Count > 0) conn.UpdateOnly(item);
                    }
                }
                //更新已关联的payment的父级或子级AmountInvoice
                if (!splitPayments.IsNullOrEmpty())
                {
                    foreach (var item in splitPayments)
                    {
                        if (item.OnlyProperties.Count > 0) conn.UpdateOnly(item);
                    }
                }
                if (!existApplies.IsNullOrEmpty())
                {
                    //删除申请关联的数据
                    var updateInvoiceApplys = conn.From<InvoiceApply>();
                    updateInvoiceApplys = updateInvoiceApplys.Update(o => new { o.Status, o.Summary });
                    updateInvoiceApplys = updateInvoiceApplys.Where(o => Sql.In(o.Kvid, existApplies.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly<InvoiceApply>(new InvoiceApply { Status = -1, Summary = $"发票作废:{Remark}" }, updateInvoiceApplys);
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    [Api(Description = "作废撤回")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceOffsetCancel : RestfulExecution<Invoice>
    {
        public Guid Kvid { get; set; }
        public List<Guid> Kvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()) throw new Exception("请传入要作废的发票Kvid!");
            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);
            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var invoices = conn.SelectByIds<Invoice>(Kvids);
            if (invoices.IsNullOrEmpty()) throw new Exception("未找到所选发票信息!");
            if (invoices.Exists(o => o.OffsetKvid == Guid.Empty)) throw new Exception("存在不正确的未作废发票,无法撤回!");
            if (invoices.Exists(o => o.Amount <= 0)) throw new Exception("请选择原作废发票信息进行撤销!");
            if (invoices.Count != Kvids.Count) throw new Exception("所选发票信息与查询后结果不一致!");
            //发票作废后 想要撤销操作,不管当初此发票是否已经有到账,都会变成借票的发票,需要冲销抵扣才行
            var offsetInvoices = conn.Select<Invoice>(o => Sql.In(o.Kvid, invoices.ConvertAll(p => p.OffsetKvid)));
            offsetInvoices.ThrowIfNullOrEmpty("未找到原作废项目!");

            var rtns = new RestfulExecutionResponse<Invoice>();
            rtns.Results = new List<Invoice>();

            var trans = conn.OpenTransaction();
            try
            {
                //原发票还原
                var updateInvoices = conn.From<Invoice>();
                updateInvoices = updateInvoices.Update(o => new { o.OffsetKvid, o.PayedTime, o.AmountPayment, o.Remark });
                updateInvoices = updateInvoices.Where(o => Sql.In(o.Kvid, invoices.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<Invoice>(new Invoice { OffsetKvid = Guid.Empty, PayedTime = null, AmountPayment = 0, Remark = string.Empty }, updateInvoices);
                foreach (var item in invoices)
                {
                    item.OffsetKvid = Guid.Empty;
                    item.PayedTime = null;
                    item.AmountPayment = 0;
                    item.Remark = string.Empty;
                    item.RemoveAllOnlyProperties();
                    rtns.Results.Add(item);
                }
                //删除Offset发票数据
                var updateOffsetInvoices = conn.From<Invoice>();
                updateOffsetInvoices = updateOffsetInvoices.Update(o => o.Status);
                updateOffsetInvoices = updateOffsetInvoices.Where(o => Sql.In(o.Kvid, offsetInvoices.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<Invoice>(new Invoice { Status = -1 }, updateOffsetInvoices);
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return base.OnExecution(req, res);
        }
    }

    //[Api(Description = "发票红冲")]
    //[RequiresAnyRole(SystemRoles.Everyone)]
    //public class InvoiceDeficit : RestfulExecution<Invoice>
    //{
    //}

    //[Api(Description = "红冲撤回")]
    //[RequiresAnyRole(SystemRoles.Everyone)]
    //public class InvoiceDeficitCancel : RestfulExecution<Invoice>
    //{

    //}
}
