﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "到账冲账")]
    //[RequiresAnyRole(SystemRoles.Administrator,SystemRoles.Manager,SystemRoles.Configuration)]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentOffset : RestfulExecution<Payment>
    {
        public Guid Kvid { get; set; }

        public List<Guid> Kvids { get; set; }

        public string Remark { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()).ThrowIfTrue("请传入要冲账的到账Kvids!");
            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payments = conn.SelectByIds<Payment>(Kvids);
            payments.ThrowIfNullOrEmpty("收付款项不存在");
            if (payments.Exists(o => o.Type != PaymentType.Bank && o.Type != PaymentType.Cash && o.Type != PaymentType.Pos && o.Type != PaymentType.AliPay && o.Type != PaymentType.WeChat)) throw new Exception("非可冲销收款类型!");
            if (payments.Exists(o => o.BizKvid != Guid.Empty)) throw new Exception("收付款已关联业务！请先解除关联！");
            if (payments.Exists(o => o.OffsetKvid != Guid.Empty)) throw new Exception("收付款已冲账");
            if (payments.Exists(o => o.Type != PaymentType.Cash && o.Type != PaymentType.Pos)) (payments.Exists(o => o.AmountInvoice > 0 || o.AmountSplited > 0 || o.AmountUsed > 0)).ThrowIfTrue($"不可冲账,存在到账已开票:{payments.Sum(o => o.AmountInvoice)}元,已认领:{payments.Sum(o => o.AmountSplited)}元,已使用:{payments.Sum(o => o.AmountUsed)}元");
            else (payments.Exists(o => o.AmountInvoice > 0 || o.AmountUsed > 0)).ThrowIfTrue($"不可冲账,存在到账已开票:{payments.Sum(o => o.AmountInvoice)}元,已使用:{payments.Sum(o => o.AmountUsed)}元");
            //查询当前项的拆分子项List
            if (conn.Exists<Payment>(o => Sql.In(o.ParentKvid, payments.ConvertAll(p => p.Kvid)) && o.OffsetKvid == Guid.Empty)) throw new Exception("不可冲账,此到账存在未冲账子项!");
            if (conn.Exists<InvoiceApply>(o => Sql.In(o.BizKvid, payments.ConvertAll(p => p.Kvid)) && o.OffsetKvid == Guid.Empty)) throw new Exception("申请开票中,不可冲账!");
            (payments.Exists(o => o.PayerAccountKvid == Guid.Empty)).ThrowIfTrue("付款账户(PayerAccountKvid)为空!");
            var allPays = conn.Select<Pay>(o => o.ToType == PayType.Payment && Sql.In(o.ToKvid, payments.ConvertAll(p => p.Kvid)));

            List<Payment> offsets = new List<Payment>();
            List<Payment> offsetPayments = new List<Payment>();
            var payOffsets = new List<Pay>();//新生成Offset的Pay
            var orginPayOffsets = new List<Pay>();//原Pay作废
            List<Account> accountPayers = conn.SelectByIds<Account>(payments.ConvertAll(p => p.PayerAccountKvid));
            foreach (var payment in payments)
            {
                var accountPayer = accountPayers.FirstOrDefault(o => o.Kvid == payment.PayerAccountKvid);
                if (accountPayer == null) continue;

                var offset = payment.Offset($"冲账:{Remark}");
                offsets.Add(offset);
                offsetPayments.Add(payment);
                if (!allPays.IsNullOrEmpty())//Transfer对应的Pay也要作废
                {
                    var currentPays = allPays.Where(o => o.ToKvid == payment.Kvid).ToList();
                    if (!currentPays.IsNullOrEmpty())
                    {
                        foreach (var pay in currentPays)
                        {
                            var offsetPay = pay.Offset($"到账冲账");
                            payOffsets.Add(offsetPay);
                            orginPayOffsets.Add(pay);
                        }
                    }
                }
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();
            try
            {
                foreach (var item in offsetPayments)
                {
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);
                    Account accountPayer = null;
                    if (!accountPayers.IsNullOrEmpty()) accountPayer = accountPayers.FirstOrDefault(o => o.Kvid == item.PayerAccountKvid);

                    if (accountPayer != null)
                    {
                        #region 新增流水
                        var accountDetail = new AccountDetail();
                        accountDetail.AccountKvid = accountPayer.Kvid;
                        accountDetail.BizId = item.SerialNumber;
                        accountDetail.BizKvid = item.Kvid;
                        accountDetail.BizType = typeof(Payment).FullName;
                        accountDetail.PayerKvid = item.PayeeKvid;
                        accountDetail.PayerName = item.PayeeName;
                        accountDetail.PayerAccountKvid = Guid.Empty;
                        accountDetail.PayerAccountName = item.PayeeAccountName;
                        accountDetail.PayerAccountSerialNumber = item.PayeeAccountSerialNumber;
                        accountDetail.PayeeKvid = item.PayerKvid;
                        accountDetail.PayeeName = item.PayerName;
                        accountDetail.PayeeAccountKvid = accountPayer.Kvid;
                        accountDetail.PayeeAccountName = accountPayer.Name;
                        accountDetail.PayeeAccountSerialNumber = accountPayer.SerialNumber;

                        accountDetail.AmountPayment = item.Amount;
                        accountDetail.Summary = "冲账:" + Remark;
                        #endregion
                        accountDetail.Insert(conn);
                    }
                }
                foreach (var item in offsets)
                {
                    conn.Insert(item);
                }
                foreach (var item in orginPayOffsets)
                {
                    conn.UpdateOnly(item);
                }
                foreach (var offset in payOffsets)
                {
                    conn.Insert(offset);
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    [Api(Description = "到账冲账撤销")]
    //[RequiresAnyRole(SystemRoles.Administrator,SystemRoles.Manager,SystemRoles.Configuration)]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentOffsetCancel : RestfulExecution<Payment>
    {
        public Guid Kvid { get; set; }

        public List<Guid> Kvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()).ThrowIfTrue("请传入要撤销冲账的到账Kvids!");
            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payments = conn.SelectByIds<Payment>(Kvids);
            payments.ThrowIfNullOrEmpty("收付款项不存在");
            if (payments.Exists(o => o.Type != PaymentType.Bank && o.Type != PaymentType.Cash && o.Type != PaymentType.Pos && o.Type != PaymentType.AliPay && o.Type != PaymentType.WeChat)) throw new Exception("不支持的操作类型!");
            (payments.Exists(o => o.BizKvid != Guid.Empty)).ThrowIfTrue("收付款已关联业务！请先解除关联！");
            (payments.Exists(o => o.Amount <= 0)).ThrowIfTrue("请选择正确的到账流水进行撤销！");
            (payments.Exists(o => o.OffsetKvid == Guid.Empty)).ThrowIfTrue("收付款未冲账,无需撤销");
            //(payments.Exists(o => o.PayeeAccountKvid == Guid.Empty)).ThrowIfTrue("收款账户(PayeeAccountKvid)为空!");
            (payments.Exists(o => o.PayerAccountKvid == Guid.Empty)).ThrowIfTrue("付款账户(PayerAccountKvid)为空!");

            var allAccountDetails = new List<AccountDetail>();
            var allAcceptPayments = new List<Payment>();
            var allAcceptPays = new List<Pay>();
            foreach (var item in payments)
            {
                //payment 取消OffsetKvid
                item.OffsetKvid = Guid.Empty;
                item.AddOnlyProperties(o => o.OffsetKvid);
                item.Remark = string.Empty;
                item.AddOnlyProperties(o => o.Remark);
                var accountDetail = item.Accept(out Pay pay);
                allAcceptPayments.Add(item);
                allAccountDetails.Add(accountDetail);
                allAcceptPays.Add(pay);
            }

            var rtns = new RestfulExecutionResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();
            try
            {
                if (!allAcceptPayments.IsNullOrEmpty())
                {
                    foreach (var item in allAcceptPayments)
                    {
                        conn.UpdateOnly(item);
                        rtns.Results.Add(item);
                    }
                }
                if (!allAccountDetails.IsNullOrEmpty()) allAccountDetails.ForEach(o => o.Insert(conn));
                if (!allAcceptPays.IsNullOrEmpty())
                {
                    foreach (var item in allAcceptPays)
                    {
                        conn.Insert(item);
                    }
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentDelete : RestfulDelete<Payment>
    {
        /// <summary>
        /// 只有作废了的到账才可以被删除
        /// </summary>
        /// <param name="req"></param>
        /// <param name="res"></param>
        /// <param name="dbConnection"></param>
        /// <param name="rtns"></param>
        /// <returns></returns>
        public override bool OnPreRestfulDelete(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulDeleteResponse<Payment> rtns)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var payments = conn.SelectByIds<Payment>(Kvids);
            payments.ThrowIfNullOrEmpty("收付款项不存在");
            (payments.Exists(o => o.OffsetKvid == Guid.Empty)).ThrowIfTrue("收付款未冲账,无需撤销");
            return base.OnPreRestfulDelete(req, res, dbConnection, rtns);
        }
    }
}
